@@ -4,20 +4,7 @@
  *
  * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+ * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
 /* BB_AUDIT SUSv3 _NOT_ compliant -- options -a, -m, -r, -t not supported. */
@@ -30,23 +17,18 @@
  * Also, exiting on a failure was a bug.  All args should be processed.
  */
 
-#include <stdio.h>
-#include <sys/types.h>
-#include <fcntl.h>
-#include <utime.h>
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include "busybox.h"
+#include "libbb.h"
 
+/* This is a NOFORK applet. Be very careful! */
+
+int touch_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int touch_main(int argc, char **argv)
 {
 	int fd;
-	int flags;
 	int status = EXIT_SUCCESS;
+	int flags = getopt32(argv, "cf");
 
-	flags = bb_getopt_ulflags(argc, argv, "c");
-
+	flags &= 1; /* ignoring -f (BSD compat thingy) */
 	argv += optind;
 
 	if (!*argv) {
@@ -55,8 +37,8 @@ int touch_main(int argc, char **argv)
 
 	do {
 		if (utime(*argv, NULL)) {
-			if (errno == ENOENT) {	/* no such file*/
-				if (flags & 1) {	/* Creation is disabled, so ignore. */
+			if (errno == ENOENT) {	/* no such file */
+				if (flags) {	/* Creation is disabled, so ignore. */
 					continue;
 				}
 				/* Try to create the file. */
@@ -68,7 +50,7 @@ int touch_main(int argc, char **argv)
 				}
 			}
 			status = EXIT_FAILURE;
-			bb_perror_msg("%s", *argv);
+			bb_simple_perror_msg(*argv);
 		}
 	} while (*++argv);
 
