@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /*
  * Support code for the hexdump and od applets,
  * based on code from util-linux v 2.11l
@@ -10,10 +11,6 @@
  * Original copyright notice is retained at the end of this file.
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <ctype.h>		/* for isdigit() */
 #include "libbb.h"
 #include "dump.h"
 
@@ -29,18 +26,18 @@ static int exitval;			/* final exit value */
 int bb_dump_blocksize;			/* data block size */
 int bb_dump_length = -1;		/* max bytes to read */
 
-static const char index_str[] = ".#-+ 0123456789";
+static const char index_str[] ALIGN1 = ".#-+ 0123456789";
 
-static const char size_conv_str[] =
+static const char size_conv_str[] ALIGN1 =
 "\x1\x4\x4\x4\x4\x4\x4\x8\x8\x8\x8\010cdiouxXeEfgG";
 
-static const char lcc[] = "diouxX";
+static const char lcc[] ALIGN1 = "diouxX";
 
 int bb_dump_size(FS * fs)
 {
-	register FU *fu;
-	register int bcnt, cur_size;
-	register char *fmt;
+	FU *fu;
+	int bcnt, cur_size;
+	char *fmt;
 	const char *p;
 	int prec;
 
@@ -62,7 +59,8 @@ int bb_dump_size(FS * fs)
 				prec = atoi(fmt);
 				while (isdigit(*++fmt));
 			}
-			if (!(p = strchr(size_conv_str + 12, *fmt))) {
+			p = strchr(size_conv_str + 12, *fmt);
+			if (!p) {
 				if (*fmt == 's') {
 					bcnt += prec;
 				} else if (*fmt == '_') {
@@ -77,15 +75,15 @@ int bb_dump_size(FS * fs)
 		}
 		cur_size += bcnt * fu->reps;
 	}
-	return (cur_size);
+	return cur_size;
 }
 
 static void rewrite(FS * fs)
 {
 	enum { NOTOKAY, USEBCNT, USEPREC } sokay;
-	register PR *pr, **nextpr = NULL;
-	register FU *fu;
-	register char *p1, *p2, *p3;
+	PR *pr, **nextpr = NULL;
+	FU *fu;
+	char *p1, *p2, *p3;
 	char savech, *fmtp;
 	const char *byte_count_str;
 	int nconv, prec = 0;
@@ -98,7 +96,7 @@ static void rewrite(FS * fs)
 		for (nconv = 0, fmtp = fu->fmt; *fmtp; nextpr = &pr->nextpr) {
 			/* NOSTRICT */
 			/* DBU:[dvae@cray.com] calloc so that forward ptrs start out NULL*/
-			pr = (PR *) xcalloc(1,sizeof(PR));
+			pr = xzalloc(sizeof(PR));
 			if (!fu->nextpr)
 				fu->nextpr = pr;
 			/* ignore nextpr -- its unused inside the loop and is
@@ -156,7 +154,7 @@ static void rewrite(FS * fs)
 				}
 				/* Unlike the original, output the remainder of the format string. */
 				if (!*byte_count_str) {
-					bb_error_msg_and_die("bad byte count for conversion character %s.", p1);
+					bb_error_msg_and_die("bad byte count for conversion character %s", p1);
 				}
 				pr->bcnt = *byte_count_str;
 			} else if (*p1 == 'l') {
@@ -165,7 +163,8 @@ static void rewrite(FS * fs)
 			DO_INT_CONV:
 				{
 					const char *e;
-					if (!(e = strchr(lcc, *p1))) {
+					e = strchr(lcc, *p1);
+					if (!e) {
 						goto DO_BAD_CONV_CHAR;
 					}
 					pr->flags = F_INT;
@@ -189,7 +188,7 @@ static void rewrite(FS * fs)
 				} else if (sokay == USEPREC) {
 					pr->bcnt = prec;
 				} else {	/* NOTOKAY */
-					bb_error_msg_and_die("%%s requires a precision or a byte count.");
+					bb_error_msg_and_die("%%s requires a precision or a byte count");
 				}
 			} else if (*p1 == '_') {
 				++p2;
@@ -223,7 +222,7 @@ static void rewrite(FS * fs)
 				}
 			} else {
 			DO_BAD_CONV_CHAR:
-				bb_error_msg_and_die("bad conversion character %%%s.\n", p1);
+				bb_error_msg_and_die("bad conversion character %%%s", p1);
 			}
 
 			/*
@@ -232,7 +231,7 @@ static void rewrite(FS * fs)
 			 */
 			savech = *p2;
 			p1[1] = '\0';
-			pr->fmt = bb_xstrdup(fmtp);
+			pr->fmt = xstrdup(fmtp);
 			*p2 = savech;
 			pr->cchar = pr->fmt + (p1 - fmtp);
 
@@ -246,8 +245,7 @@ static void rewrite(FS * fs)
 			{
 				savech = *p3;
 				*p3 = '\0';
-				if (!(pr->fmt = realloc(pr->fmt, strlen(pr->fmt)+(p3-p2)+1)))
-					bb_perror_msg_and_die("hexdump");
+				pr->fmt = xrealloc(pr->fmt, strlen(pr->fmt)+(p3-p2)+1);
 				strcat(pr->fmt, p2);
 				*p3 = savech;
 				p2 = p3;
@@ -257,7 +255,7 @@ static void rewrite(FS * fs)
 
 			/* only one conversion character if byte count */
 			if (!(pr->flags & F_ADDRESS) && fu->bcnt && nconv++) {
-				bb_error_msg_and_die("byte count with multiple conversion characters.\n");
+				bb_error_msg_and_die("byte count with multiple conversion characters");
 			}
 		}
 		/*
@@ -295,13 +293,13 @@ static void rewrite(FS * fs)
 	}
 }
 
-static void do_skip(char *fname, int statok)
+static void do_skip(const char *fname, int statok)
 {
 	struct stat sbuf;
 
 	if (statok) {
 		if (fstat(STDIN_FILENO, &sbuf)) {
-			bb_perror_msg_and_die("%s", fname);
+			bb_simple_perror_msg_and_die(fname);
 		}
 		if ((!(S_ISCHR(sbuf.st_mode) ||
 			   S_ISBLK(sbuf.st_mode) ||
@@ -313,7 +311,7 @@ static void do_skip(char *fname, int statok)
 		}
 	}
 	if (fseek(stdin, bb_dump_skip, SEEK_SET)) {
-		bb_perror_msg_and_die("%s", fname);
+		bb_simple_perror_msg_and_die(fname);
 	}
 	savaddress = address += bb_dump_skip;
 	bb_dump_skip = 0;
@@ -321,25 +319,27 @@ static void do_skip(char *fname, int statok)
 
 static int next(char **argv)
 {
-	static int done;
+	static smallint done;
+
 	int statok;
 
 	if (argv) {
 		_argv = argv;
-		return (1);
+		return 1;
 	}
 	for (;;) {
 		if (*_argv) {
 			if (!(freopen(*_argv, "r", stdin))) {
-				bb_perror_msg("%s", *_argv);
+				bb_simple_perror_msg(*_argv);
 				exitval = 1;
 				++_argv;
 				continue;
 			}
-			statok = done = 1;
+			done = statok = 1;
 		} else {
-			if (done++)
-				return (0);
+			if (done)
+				return 0;
+			done = 1;
 			statok = 0;
 		}
 		if (bb_dump_skip)
@@ -347,23 +347,24 @@ static int next(char **argv)
 		if (*_argv)
 			++_argv;
 		if (!bb_dump_skip)
-			return (1);
+			return 1;
 	}
 	/* NOTREACHED */
 }
 
 static unsigned char *get(void)
 {
-	static int ateof = 1;
-	static unsigned char *curp=NULL, *savp; /*DBU:[dave@cray.com]initialize curp */
-	register int n;
+	static smallint ateof = 1;
+	static unsigned char *curp = NULL, *savp; /*DBU:[dave@cray.com]initialize curp */
+
+	int n;
 	int need, nread;
 	unsigned char *tmpp;
 
 	if (!curp) {
 		address = (off_t)0; /*DBU:[dave@cray.com] initialize,initialize..*/
-		curp = (unsigned char *) xmalloc(bb_dump_blocksize);
-		savp = (unsigned char *) xmalloc(bb_dump_blocksize);
+		curp = xmalloc(bb_dump_blocksize);
+		savp = xmalloc(bb_dump_blocksize);
 	} else {
 		tmpp = curp;
 		curp = savp;
@@ -378,23 +379,23 @@ static unsigned char *get(void)
 		 */
 		if (!bb_dump_length || (ateof && !next((char **) NULL))) {
 			if (need == bb_dump_blocksize) {
-				return ((unsigned char *) NULL);
+				return NULL;
 			}
-			if (bb_dump_vflag != ALL && !bcmp(curp, savp, nread)) {
+			if (bb_dump_vflag != ALL && !memcmp(curp, savp, nread)) {
 				if (bb_dump_vflag != DUP) {
-					printf("*\n");
+					puts("*");
 				}
-				return ((unsigned char *) NULL);
+				return NULL;
 			}
 			memset((char *) curp + nread, 0, need);
 			eaddress = address + nread;
-			return (curp);
+			return curp;
 		}
 		n = fread((char *) curp + nread, sizeof(unsigned char),
 				  bb_dump_length == -1 ? need : MIN(bb_dump_length, need), stdin);
 		if (!n) {
 			if (ferror(stdin)) {
-				bb_perror_msg("%s", _argv[-1]);
+				bb_simple_perror_msg(_argv[-1]);
 			}
 			ateof = 1;
 			continue;
@@ -403,16 +404,17 @@ static unsigned char *get(void)
 		if (bb_dump_length != -1) {
 			bb_dump_length -= n;
 		}
-		if (!(need -= n)) {
+		need -= n;
+		if (!need) {
 			if (bb_dump_vflag == ALL || bb_dump_vflag == FIRST
-				|| bcmp(curp, savp, bb_dump_blocksize)) {
+				|| memcmp(curp, savp, bb_dump_blocksize)) {
 				if (bb_dump_vflag == DUP || bb_dump_vflag == FIRST) {
 					bb_dump_vflag = WAIT;
 				}
-				return (curp);
+				return curp;
 			}
 			if (bb_dump_vflag == WAIT) {
-				printf("*\n");
+				puts("*");
 			}
 			bb_dump_vflag = DUP;
 			address = savaddress += bb_dump_blocksize;
@@ -426,7 +428,7 @@ static unsigned char *get(void)
 
 static void bpad(PR * pr)
 {
-	register char *p1, *p2;
+	char *p1, *p2;
 
 	/*
 	 * remove all conversion flags; '-' is the only one valid
@@ -435,11 +437,12 @@ static void bpad(PR * pr)
 	pr->flags = F_BPAD;
 	*pr->cchar = 's';
 	for (p1 = pr->fmt; *p1 != '%'; ++p1);
-	for (p2 = ++p1; *p1 && strchr(" -0+#", *p1); ++p1);
+	for (p2 = ++p1; *p1 && strchr(" -0+#", *p1); ++p1)
+		if (pr->nospace) pr->nospace--;
 	while ((*p2++ = *p1++) != 0);
 }
 
-static const char conv_str[] =
+static const char conv_str[] ALIGN1 =
 	"\0\\0\0"
 	"\007\\a\0"				/* \a */
 	"\b\\b\0"
@@ -448,7 +451,7 @@ static const char conv_str[] =
 	"\r\\r\0"
 	"\t\\t\0"
 	"\v\\v\0"
-	"\0";
+	;
 
 
 static void conv_c(PR * pr, unsigned char * p)
@@ -478,7 +481,7 @@ static void conv_c(PR * pr, unsigned char * p)
 
 static void conv_u(PR * pr, unsigned char * p)
 {
-	static const char list[] =
+	static const char list[] ALIGN1 =
 		"nul\0soh\0stx\0etx\0eot\0enq\0ack\0bel\0"
 		"bs\0_ht\0_lf\0_vt\0_ff\0_cr\0_so\0_si\0_"
 		"dle\0dcl\0dc2\0dc3\0dc4\0nak\0syn\0etb\0"
@@ -503,11 +506,11 @@ static void conv_u(PR * pr, unsigned char * p)
 static void display(void)
 {
 /*  extern FU *endfu; */
-	register FS *fs;
-	register FU *fu;
-	register PR *pr;
-	register int cnt;
-	register unsigned char *bp;
+	FS *fs;
+	FU *fu;
+	PR *pr;
+	int cnt;
+	unsigned char *bp;
 
 	off_t saveaddress;
 	unsigned char savech = 0, *savebp;
@@ -550,12 +553,12 @@ static void display(void)
 
 							switch (pr->bcnt) {
 							case 4:
-								bcopy((char *) bp, (char *) &fval,
+								memmove((char *) &fval, (char *) bp,
 									  sizeof(fval));
 								printf(pr->fmt, fval);
 								break;
 							case 8:
-								bcopy((char *) bp, (char *) &dval,
+								memmove((char *) &dval, (char *) bp,
 									  sizeof(dval));
 								printf(pr->fmt, dval);
 								break;
@@ -571,12 +574,12 @@ static void display(void)
 								printf(pr->fmt, (int) *bp);
 								break;
 							case 2:
-								bcopy((char *) bp, (char *) &sval,
+								memmove((char *) &sval, (char *) bp,
 									  sizeof(sval));
 								printf(pr->fmt, (int) sval);
 								break;
 							case 4:
-								bcopy((char *) bp, (char *) &ival,
+								memmove((char *) &ival, (char *) bp,
 									  sizeof(ival));
 								printf(pr->fmt, ival);
 								break;
@@ -604,12 +607,12 @@ static void display(void)
 								printf(pr->fmt, (unsigned int) * bp);
 								break;
 							case 2:
-								bcopy((char *) bp, (char *) &sval,
+								memmove((char *) &sval, (char *) bp,
 									  sizeof(sval));
 								printf(pr->fmt, (unsigned int) sval);
 								break;
 							case 4:
-								bcopy((char *) bp, (char *) &ival,
+								memmove((char *) &ival, (char *) bp,
 									  sizeof(ival));
 								printf(pr->fmt, ival);
 								break;
@@ -651,7 +654,7 @@ static void display(void)
 
 int bb_dump_dump(char **argv)
 {
-	register FS *tfs;
+	FS *tfs;
 
 	/* figure out the data block bb_dump_size */
 	for (bb_dump_blocksize = 0, tfs = bb_dump_fshead; tfs; tfs = tfs->nextfs) {
@@ -668,22 +671,21 @@ int bb_dump_dump(char **argv)
 	next(argv);
 	display();
 
-	return (exitval);
+	return exitval;
 }
 
 void bb_dump_add(const char *fmt)
 {
-	register const char *p;
-	register char *p1;
-	register char *p2;
+	const char *p;
+	char *p1;
+	char *p2;
 	static FS **nextfs;
 	FS *tfs;
 	FU *tfu, **nextfu;
 	const char *savep;
 
 	/* start new linked list of format units */
-	/* NOSTRICT */
-	tfs = (FS *) xcalloc(1,sizeof(FS)); /*DBU:[dave@cray.com] start out NULL */
+	tfs = xzalloc(sizeof(FS)); /*DBU:[dave@cray.com] start out NULL */
 	if (!bb_dump_fshead) {
 		bb_dump_fshead = tfs;
 	} else {
@@ -695,7 +697,7 @@ void bb_dump_add(const char *fmt)
 	/* take the format string and break it up into format units */
 	for (p = fmt;;) {
 		/* bb_dump_skip leading white space */
-		p = bb_skip_whitespace(p);
+		p = skip_whitespace(p);
 		if (!*p) {
 			break;
 		}
@@ -703,7 +705,7 @@ void bb_dump_add(const char *fmt)
 		/* allocate a new format unit and link it in */
 		/* NOSTRICT */
 		/* DBU:[dave@cray.com] calloc so that forward pointers start out NULL */
-		tfu = (FU *) xcalloc(1,sizeof(FU));
+		tfu = xzalloc(sizeof(FU));
 		*nextfu = tfu;
 		nextfu = &tfu->nextfu;
 		tfu->reps = 1;
@@ -718,23 +720,25 @@ void bb_dump_add(const char *fmt)
 			tfu->reps = atoi(savep);
 			tfu->flags = F_SETREP;
 			/* bb_dump_skip trailing white space */
-			p = bb_skip_whitespace(++p);
+			p = skip_whitespace(++p);
 		}
 
 		/* bb_dump_skip slash and trailing white space */
 		if (*p == '/') {
-			p = bb_skip_whitespace(++p);
+			p = skip_whitespace(++p);
 		}
 
 		/* byte count */
 		if (isdigit(*p)) {
-			for (savep = p; isdigit(*p); ++p);
+// TODO: use bb_strtou
+			savep = p;
+			do p++; while (isdigit(*p));
 			if (!isspace(*p)) {
 				bb_error_msg_and_die("bad format {%s}", fmt);
 			}
 			tfu->bcnt = atoi(savep);
 			/* bb_dump_skip trailing white space */
-			p = bb_skip_whitespace(++p);
+			p = skip_whitespace(++p);
 		}
 
 		/* format */
