@@ -13,25 +13,16 @@
  * Kunihiro Ishiguro <kunihiro@zebra.org> 001102: rtnh_ifindex was not initialized
  */
 
-#include <sys/socket.h>
-
-#include <stdlib.h>
-#include <string.h>
-#include <fcntl.h>
-#include <unistd.h>
-
+#include "ip_common.h"	/* #include "libbb.h" is inside */
 #include "rt_names.h"
 #include "utils.h"
 
-#include "libbb.h"
-
 #ifndef RTAX_RTTVAR
 #define RTAX_RTTVAR RTAX_HOPS
 #endif
 
 
-static struct
-{
+typedef struct filter_t {
 	int tb;
 	int flushed;
 	char *flushb;
@@ -51,18 +42,42 @@ static struct
 	inet_prefix mdst;
 	inet_prefix rsrc;
 	inet_prefix msrc;
-} filter;
+} filter_t;
+
+#define filter (*(filter_t*)&bb_common_bufsiz1)
 
 static int flush_update(void)
 {
 	if (rtnl_send(filter.rth, filter.flushb, filter.flushp) < 0) {
-		perror("Failed to send flush request\n");
+		bb_perror_msg("failed to send flush request");
 		return -1;
 	}
 	filter.flushp = 0;
 	return 0;
 }
 
+static unsigned get_hz(void)
+{
+	static unsigned hz_internal;
+	FILE *fp;
+
+	if (hz_internal)
+		return hz_internal;
+
+	fp = fopen("/proc/net/psched", "r");
+	if (fp) {
+		unsigned nom, denom;
+
+		if (fscanf(fp, "%*08x%*08x%08x%08x", &nom, &denom) == 2)
+			if (nom == 1000000)
+				hz_internal = denom;
+		fclose(fp);
+	}
+	if (!hz_internal)
+		hz_internal = sysconf(_SC_CLK_TCK);
+	return hz_internal;
+}
+
 static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 		struct nlmsghdr *n, void *arg)
 {
@@ -85,10 +100,8 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 	if (filter.flushb && n->nlmsg_type != RTM_NEWROUTE)
 		return 0;
 	len -= NLMSG_LENGTH(sizeof(*r));
-	if (len < 0) {
-		bb_error_msg("wrong nlmsg len %d", len);
-		return -1;
-	}
+	if (len < 0)
+		bb_error_msg_and_die("wrong nlmsg len %d", len);
 
 	if (r->rtm_family == AF_INET6)
 		host_len = 128;
@@ -98,11 +111,11 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 	if (r->rtm_family == AF_INET6) {
 		if (filter.tb) {
 			if (filter.tb < 0) {
-				if (!(r->rtm_flags&RTM_F_CLONED)) {
+				if (!(r->rtm_flags & RTM_F_CLONED)) {
 					return 0;
 				}
 			} else {
-				if (r->rtm_flags&RTM_F_CLONED) {
+				if (r->rtm_flags & RTM_F_CLONED) {
 					return 0;
 				}
 				if (filter.tb == RT_TABLE_LOCAL) {
@@ -169,7 +182,7 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 		struct nlmsghdr *fn;
 		if (NLMSG_ALIGN(filter.flushp) + n->nlmsg_len > filter.flushe) {
 			if (flush_update())
-				return -1;
+				bb_error_msg_and_die("flush");
 		}
 		fn = (struct nlmsghdr*)(filter.flushb + NLMSG_ALIGN(filter.flushp));
 		memcpy(fn, n, n->nlmsg_len);
@@ -248,7 +261,7 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 				    abuf, sizeof(abuf)));
 	}
 	if (tb[RTA_PRIORITY]) {
-		fprintf(fp, " metric %d ", *(__u32*)RTA_DATA(tb[RTA_PRIORITY]));
+		fprintf(fp, " metric %d ", *(uint32_t*)RTA_DATA(tb[RTA_PRIORITY]));
 	}
 	if (r->rtm_family == AF_INET6) {
 		struct rta_cacheinfo *ci = NULL;
@@ -256,15 +269,11 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 			ci = RTA_DATA(tb[RTA_CACHEINFO]);
 		}
 		if ((r->rtm_flags & RTM_F_CLONED) || (ci && ci->rta_expires)) {
-			static int hz;
-			if (!hz) {
-				hz = get_hz();
-			}
 			if (r->rtm_flags & RTM_F_CLONED) {
-				fprintf(fp, "%s    cache ", _SL_);
+				fprintf(fp, "%c    cache ", _SL_);
 			}
 			if (ci->rta_expires) {
-				fprintf(fp, " expires %dsec", ci->rta_expires/hz);
+				fprintf(fp, " expires %dsec", ci->rta_expires / get_hz());
 			}
 			if (ci->rta_error != 0) {
 				fprintf(fp, " error %d", ci->rta_error);
@@ -277,27 +286,45 @@ static int print_route(struct sockaddr_nl *who ATTRIBUTE_UNUSED,
 	if (tb[RTA_IIF] && filter.iifmask != -1) {
 		fprintf(fp, " iif %s", ll_index_to_name(*(int*)RTA_DATA(tb[RTA_IIF])));
 	}
-	fprintf(fp, "\n");
+	fputc('\n', fp);
 	fflush(fp);
 	return 0;
 }
 
-static int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
+/* Return value becomes exitcode. It's okay to not return at all */
+static int iproute_modify(int cmd, unsigned flags, char **argv)
 {
+	static const char keywords[] ALIGN1 =
+		"src\0""via\0""mtu\0""lock\0""protocol\0"USE_FEATURE_IP_RULE("table\0")
+		"dev\0""oif\0""to\0";
+	enum {
+		ARG_src,
+		ARG_via,
+		ARG_mtu, PARM_lock,
+		ARG_protocol,
+USE_FEATURE_IP_RULE(ARG_table,)
+		ARG_dev,
+		ARG_oif,
+		ARG_to
+	};
+	enum {
+		gw_ok = 1 << 0,
+		dst_ok = 1 << 1,
+		proto_ok = 1 << 2,
+		type_ok = 1 << 3
+	};
 	struct rtnl_handle rth;
 	struct {
 		struct nlmsghdr		n;
 		struct rtmsg		r;
 		char			buf[1024];
 	} req;
-	char  mxbuf[256];
+	char mxbuf[256];
 	struct rtattr * mxrta = (void*)mxbuf;
 	unsigned mxlock = 0;
-	char  *d = NULL;
-	int gw_ok = 0;
-	int dst_ok = 0;
-	int proto_ok = 0;
-	int type_ok = 0;
+	char *d = NULL;
+	smalluint ok = 0;
+	int arg;
 
 	memset(&req, 0, sizeof(req));
 
@@ -317,61 +344,67 @@ static int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 	mxrta->rta_type = RTA_METRICS;
 	mxrta->rta_len = RTA_LENGTH(0);
 
-	while (argc > 0) {
-		if (strcmp(*argv, "src") == 0) {
+	while (*argv) {
+		arg = index_in_substrings(keywords, *argv);
+		if (arg == ARG_src) {
 			inet_prefix addr;
 			NEXT_ARG();
 			get_addr(&addr, *argv, req.r.rtm_family);
-			if (req.r.rtm_family == AF_UNSPEC) {
+			if (req.r.rtm_family == AF_UNSPEC)
 				req.r.rtm_family = addr.family;
-			}
 			addattr_l(&req.n, sizeof(req), RTA_PREFSRC, &addr.data, addr.bytelen);
-		} else if (strcmp(*argv, "via") == 0) {
+		} else if (arg == ARG_via) {
 			inet_prefix addr;
-			gw_ok = 1;
+			ok |= gw_ok;
 			NEXT_ARG();
 			get_addr(&addr, *argv, req.r.rtm_family);
 			if (req.r.rtm_family == AF_UNSPEC) {
 				req.r.rtm_family = addr.family;
 			}
 			addattr_l(&req.n, sizeof(req), RTA_GATEWAY, &addr.data, addr.bytelen);
-		} else if (strcmp(*argv, "mtu") == 0) {
+		} else if (arg == ARG_mtu) {
 			unsigned mtu;
 			NEXT_ARG();
-			if (strcmp(*argv, "lock") == 0) {
-				mxlock |= (1<<RTAX_MTU);
+			if (index_in_strings(keywords, *argv) == PARM_lock) {
+				mxlock |= (1 << RTAX_MTU);
 				NEXT_ARG();
 			}
-			if (get_unsigned(&mtu, *argv, 0)) {
-				invarg("\"mtu\" value is invalid\n", *argv);
-			}
+			if (get_unsigned(&mtu, *argv, 0))
+				invarg(*argv, "mtu");
 			rta_addattr32(mxrta, sizeof(mxbuf), RTAX_MTU, mtu);
-		} else if (matches(*argv, "protocol") == 0) {
+		} else if (arg == ARG_protocol) {
 			uint32_t prot;
 			NEXT_ARG();
 			if (rtnl_rtprot_a2n(&prot, *argv))
-				invarg("\"protocol\" value is invalid\n", *argv);
+				invarg(*argv, "protocol");
 			req.r.rtm_protocol = prot;
-			proto_ok =1;
-		} else if (strcmp(*argv, "dev") == 0 ||
-			   strcmp(*argv, "oif") == 0) {
+			ok |= proto_ok;
+#if ENABLE_FEATURE_IP_RULE
+		} else if (arg == ARG_table) {
+			uint32_t tid;
+			NEXT_ARG();
+			if (rtnl_rttable_a2n(&tid, *argv))
+				invarg(*argv, "table");
+			req.r.rtm_table = tid;
+#endif
+		} else if (arg == ARG_dev || arg == ARG_oif) {
 			NEXT_ARG();
 			d = *argv;
 		} else {
 			int type;
 			inet_prefix dst;
 
-			if (strcmp(*argv, "to") == 0) {
+			if (arg == ARG_to) {
 				NEXT_ARG();
 			}
-			if ((**argv < '0' || **argv > '9') &&
-			    rtnl_rtntype_a2n(&type, *argv) == 0) {
+			if ((**argv < '0' || **argv > '9')
+			 && rtnl_rtntype_a2n(&type, *argv) == 0) {
 				NEXT_ARG();
 				req.r.rtm_type = type;
-				type_ok = 1;
+				ok |= type_ok;
 			}
 
-			if (dst_ok) {
+			if (ok & dst_ok) {
 				duparg2("to", *argv);
 			}
 			get_prefix(&dst, *argv, req.r.rtm_family);
@@ -379,17 +412,15 @@ static int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 				req.r.rtm_family = dst.family;
 			}
 			req.r.rtm_dst_len = dst.bitlen;
-			dst_ok = 1;
+			ok |= dst_ok;
 			if (dst.bytelen) {
 				addattr_l(&req.n, sizeof(req), RTA_DST, &dst.data, dst.bytelen);
 			}
 		}
-		argc--; argv++;
+		argv++;
 	}
 
-	if (rtnl_open(&rth, 0) < 0) {
-		exit(1);
-	}
+	xrtnl_open(&rth);
 
 	if (d)  {
 		int idx;
@@ -397,10 +428,7 @@ static int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 		ll_init_map(&rth);
 
 		if (d) {
-			if ((idx = ll_name_to_index(d)) == 0) {
-				bb_error_msg("Cannot find device \"%s\"", d);
-				return -1;
-			}
+			idx = xll_name_to_index(d);
 			addattr32(&req.n, sizeof(req), RTA_OIF, idx);
 		}
 	}
@@ -412,12 +440,25 @@ static int iproute_modify(int cmd, unsigned flags, int argc, char **argv)
 		addattr_l(&req.n, sizeof(req), RTA_METRICS, RTA_DATA(mxrta), RTA_PAYLOAD(mxrta));
 	}
 
+	if (req.r.rtm_type == RTN_LOCAL || req.r.rtm_type == RTN_NAT)
+		req.r.rtm_scope = RT_SCOPE_HOST;
+	else if (req.r.rtm_type == RTN_BROADCAST ||
+			req.r.rtm_type == RTN_MULTICAST ||
+			req.r.rtm_type == RTN_ANYCAST)
+		req.r.rtm_scope = RT_SCOPE_LINK;
+	else if (req.r.rtm_type == RTN_UNICAST || req.r.rtm_type == RTN_UNSPEC) {
+		if (cmd == RTM_DELROUTE)
+			req.r.rtm_scope = RT_SCOPE_NOWHERE;
+		else if (!(ok & gw_ok))
+			req.r.rtm_scope = RT_SCOPE_LINK;
+	}
+
 	if (req.r.rtm_family == AF_UNSPEC) {
 		req.r.rtm_family = AF_INET;
 	}
 
 	if (rtnl_talk(&rth, &req.n, 0, 0, NULL, NULL, NULL) < 0) {
-		exit(2);
+		return 2;
 	}
 
 	return 0;
@@ -443,30 +484,23 @@ static int rtnl_rtcache_request(struct rtnl_handle *rth, int family)
 	req.rtm.rtm_family = family;
 	req.rtm.rtm_flags |= RTM_F_CLONED;
 
-	return sendto(rth->fd, (void*)&req, sizeof(req), 0, (struct sockaddr*)&nladdr, sizeof(nladdr));
+	return xsendto(rth->fd, (void*)&req, sizeof(req), (struct sockaddr*)&nladdr, sizeof(nladdr));
 }
 
-static int iproute_flush_cache(void)
+static void iproute_flush_cache(void)
 {
-#define ROUTE_FLUSH_PATH "/proc/sys/net/ipv4/route/flush"
-
-	int len;
-	int flush_fd = open (ROUTE_FLUSH_PATH, O_WRONLY);
-	char *buffer = "-1";
+	static const char fn[] ALIGN1 = "/proc/sys/net/ipv4/route/flush";
+	int flush_fd = open_or_warn(fn, O_WRONLY);
 
 	if (flush_fd < 0) {
-		fprintf (stderr, "Cannot open \"%s\"\n", ROUTE_FLUSH_PATH);
-		return -1;
+		return;
 	}
 
-	len = strlen (buffer);
-
-	if ((write (flush_fd, (void *)buffer, len)) < len) {
-		fprintf (stderr, "Cannot flush routing cache\n");
-		return -1;
+	if (write(flush_fd, "-1", 2) < 2) {
+		bb_perror_msg("cannot flush routing cache");
+		return;
 	}
 	close(flush_fd);
-	return 0;
 }
 
 static void iproute_reset_filter(void)
@@ -476,121 +510,142 @@ static void iproute_reset_filter(void)
 	filter.msrc.bitlen = -1;
 }
 
-static int iproute_list_or_flush(int argc, char **argv, int flush)
+/* Return value becomes exitcode. It's okay to not return at all */
+static int iproute_list_or_flush(char **argv, int flush)
 {
 	int do_ipv6 = preferred_family;
 	struct rtnl_handle rth;
 	char *id = NULL;
 	char *od = NULL;
+	static const char keywords[] ALIGN1 =
+		/* "ip route list/flush" parameters: */
+		"protocol\0" "dev\0"   "oif\0"   "iif\0"
+		"via\0"      "table\0" "cache\0"
+		"from\0"     "to\0"
+		/* and possible further keywords */
+		"all\0"
+		"root\0"
+		"match\0"
+		"exact\0"
+		"main\0"
+		;
+	enum {
+		KW_proto, KW_dev,   KW_oif,  KW_iif,
+		KW_via,   KW_table, KW_cache,
+		KW_from,  KW_to,
+		/* */
+		KW_all,
+		KW_root,
+		KW_match,
+		KW_exact,
+		KW_main,
+	};
+	int arg, parm;
 
 	iproute_reset_filter();
 	filter.tb = RT_TABLE_MAIN;
 
-	if (flush && argc <= 0) {
-		fprintf(stderr, "\"ip route flush\" requires arguments.\n");
-		return -1;
-	}
+	if (flush && !*argv)
+		bb_error_msg_and_die(bb_msg_requires_arg, "\"ip route flush\"");
 
-	while (argc > 0) {
-		if (matches(*argv, "protocol") == 0) {
+	while (*argv) {
+		arg = index_in_substrings(keywords, *argv);
+		if (arg == KW_proto) {
 			uint32_t prot = 0;
 			NEXT_ARG();
 			filter.protocolmask = -1;
 			if (rtnl_rtprot_a2n(&prot, *argv)) {
-				if (strcmp(*argv, "all") != 0) {
-					invarg("invalid \"protocol\"\n", *argv);
-				}
+				if (index_in_strings(keywords, *argv) != KW_all)
+					invarg(*argv, "protocol");
 				prot = 0;
 				filter.protocolmask = 0;
 			}
 			filter.protocol = prot;
-		} else if (strcmp(*argv, "dev") == 0 ||
-			   strcmp(*argv, "oif") == 0) {
+		} else if (arg == KW_dev || arg == KW_oif) {
 			NEXT_ARG();
 			od = *argv;
-		} else if (strcmp(*argv, "iif") == 0) {
+		} else if (arg == KW_iif) {
 			NEXT_ARG();
 			id = *argv;
-		} else if (matches(*argv, "from") == 0) {
+		} else if (arg == KW_via) {
 			NEXT_ARG();
-			if (matches(*argv, "root") == 0) {
+			get_prefix(&filter.rvia, *argv, do_ipv6);
+		} else if (arg == KW_table) { /* table all/cache/main */
+			NEXT_ARG();
+			parm = index_in_substrings(keywords, *argv);
+			if (parm == KW_cache)
+				filter.tb = -1;
+			else if (parm == KW_all)
+				filter.tb = 0;
+			else if (parm != KW_main)
+				invarg(*argv, "table");
+		} else if (arg == KW_cache) {
+			/* The command 'ip route flush cache' is used by OpenSWAN.
+			 * Assuming it's a synonym for 'ip route flush table cache' */
+			filter.tb = -1;
+		} else if (arg == KW_from) {
+			NEXT_ARG();
+			parm = index_in_substrings(keywords, *argv);
+			if (parm == KW_root) {
 				NEXT_ARG();
 				get_prefix(&filter.rsrc, *argv, do_ipv6);
-			} else if (matches(*argv, "match") == 0) {
+			} else if (parm == KW_match) {
 				NEXT_ARG();
 				get_prefix(&filter.msrc, *argv, do_ipv6);
 			} else {
-				if (matches(*argv, "exact") == 0) {
+				if (parm == KW_exact)
 					NEXT_ARG();
-				}
 				get_prefix(&filter.msrc, *argv, do_ipv6);
 				filter.rsrc = filter.msrc;
 			}
-		} else {
-			if (matches(*argv, "to") == 0) {
+		} else { /* "to" is the default parameter */
+			if (arg == KW_to) {
 				NEXT_ARG();
+				arg = index_in_substrings(keywords, *argv);
 			}
-			if (matches(*argv, "root") == 0) {
+			/* parm = arg; - would be more plausible, but we reuse 'arg' here */
+			if (arg == KW_root) {
 				NEXT_ARG();
 				get_prefix(&filter.rdst, *argv, do_ipv6);
-			} else if (matches(*argv, "match") == 0) {
+			} else if (arg == KW_match) {
 				NEXT_ARG();
 				get_prefix(&filter.mdst, *argv, do_ipv6);
-			} else if (matches(*argv, "table") == 0) {
-				NEXT_ARG();
-				if (matches(*argv, "cache") == 0) {
-					filter.tb = -1;
-				} else if (matches(*argv, "main") != 0) {
-					invarg("invalid \"table\"", *argv);
-				}
-			} else if (matches(*argv, "cache") == 0) {
-				filter.tb = -1;
-			} else {
-				if (matches(*argv, "exact") == 0) {
+			} else { /* "to exact" is the default */
+				if (arg == KW_exact)
 					NEXT_ARG();
-				}
 				get_prefix(&filter.mdst, *argv, do_ipv6);
 				filter.rdst = filter.mdst;
 			}
 		}
-		argc--; argv++;
+		argv++;
 	}
 
 	if (do_ipv6 == AF_UNSPEC && filter.tb) {
 		do_ipv6 = AF_INET;
 	}
 
-	if (rtnl_open(&rth, 0) < 0) {
-		exit(1);
-	}
-
+	xrtnl_open(&rth);
 	ll_init_map(&rth);
 
 	if (id || od)  {
 		int idx;
 
 		if (id) {
-			if ((idx = ll_name_to_index(id)) == 0) {
-				bb_error_msg("Cannot find device \"%s\"", id);
-				return -1;
-			}
+			idx = xll_name_to_index(id);
 			filter.iif = idx;
 			filter.iifmask = -1;
 		}
 		if (od) {
-			if ((idx = ll_name_to_index(od)) == 0) {
-				bb_error_msg("Cannot find device \"%s\"", od);
-			}
+			idx = xll_name_to_index(od);
 			filter.oif = idx;
 			filter.oifmask = -1;
 		}
 	}
 
 	if (flush) {
-		int _round = 0;
 		char flushb[4096-512];
 
-		if (filter.tb == -1) {
+		if (filter.tb == -1) { /* "flush table cache" */
 			if (do_ipv6 != AF_INET6)
 				iproute_flush_cache();
 			if (do_ipv6 == AF_INET)
@@ -603,61 +658,42 @@ static int iproute_list_or_flush(int argc, char **argv, int flush)
 		filter.rth = &rth;
 
 		for (;;) {
-			if (rtnl_wilddump_request(&rth, do_ipv6, RTM_GETROUTE) < 0) {
-				perror("Cannot send dump request");
-				return -1;
-			}
+			xrtnl_wilddump_request(&rth, do_ipv6, RTM_GETROUTE);
 			filter.flushed = 0;
-			if (rtnl_dump_filter(&rth, print_route, stdout, NULL, NULL) < 0) {
-				bb_error_msg("Flush terminated\n");
-				return -1;
-			}
-			if (filter.flushed == 0) {
-				if (_round == 0) {
-					if (filter.tb != -1 || do_ipv6 == AF_INET6)
-						fprintf(stderr, "Nothing to flush.\n");
-				}
-				fflush(stdout);
+			xrtnl_dump_filter(&rth, print_route, stdout);
+			if (filter.flushed == 0)
 				return 0;
-			}
-			_round++;
-			if (flush_update() < 0)
-				exit(1);
+			if (flush_update())
+				return 1;
 		}
 	}
 
 	if (filter.tb != -1) {
-		if (rtnl_wilddump_request(&rth, do_ipv6, RTM_GETROUTE) < 0) {
-			bb_perror_msg_and_die("Cannot send dump request");
-		}
-	} else {
-		if (rtnl_rtcache_request(&rth, do_ipv6) < 0) {
-			bb_perror_msg_and_die("Cannot send dump request");
-		}
+		xrtnl_wilddump_request(&rth, do_ipv6, RTM_GETROUTE);
+	} else if (rtnl_rtcache_request(&rth, do_ipv6) < 0) {
+		bb_perror_msg_and_die("cannot send dump request");
 	}
+	xrtnl_dump_filter(&rth, print_route, stdout);
 
-	if (rtnl_dump_filter(&rth, print_route, stdout, NULL, NULL) < 0) {
-		bb_error_msg_and_die("Dump terminated");
-	}
-
-	exit(0);
+	return 0;
 }
 
 
-static int iproute_get(int argc, char **argv)
+/* Return value becomes exitcode. It's okay to not return at all */
+static int iproute_get(char **argv)
 {
 	struct rtnl_handle rth;
 	struct {
-		struct nlmsghdr		n;
-		struct rtmsg		r;
-		char			buf[1024];
+		struct nlmsghdr n;
+		struct rtmsg    r;
+		char            buf[1024];
 	} req;
-	char  *idev = NULL;
-	char  *odev = NULL;
-	int connected = 0;
-	int from_ok = 0;
-	static const char * const options[] =
-		{ "from", "iif", "oif", "dev", "notify", "connected", "to", 0 };
+	char *idev = NULL;
+	char *odev = NULL;
+	bool connected = 0;
+	bool from_ok = 0;
+	static const char options[] ALIGN1 =
+		"from\0""iif\0""oif\0""dev\0""notify\0""connected\0""to\0";
 
 	memset(&req, 0, sizeof(req));
 
@@ -675,8 +711,8 @@ static int iproute_get(int argc, char **argv)
 	req.r.rtm_dst_len = 0;
 	req.r.rtm_tos = 0;
 
-	while (argc > 0) {
-		switch (compare_string_array(options, *argv)) {
+	while (*argv) {
+		switch (index_in_strings(options, *argv)) {
 			case 0: /* from */
 			{
 				inet_prefix addr;
@@ -721,7 +757,7 @@ static int iproute_get(int argc, char **argv)
 				}
 				req.r.rtm_dst_len = addr.bitlen;
 			}
-			argc--; argv++;
+			argv++;
 		}
 	}
 
@@ -729,8 +765,7 @@ static int iproute_get(int argc, char **argv)
 		bb_error_msg_and_die("need at least destination address");
 	}
 
-	if (rtnl_open(&rth, 0) < 0)
-		exit(1);
+	xrtnl_open(&rth);
 
 	ll_init_map(&rth);
 
@@ -738,17 +773,11 @@ static int iproute_get(int argc, char **argv)
 		int idx;
 
 		if (idev) {
-			if ((idx = ll_name_to_index(idev)) == 0) {
-				bb_error_msg("Cannot find device \"%s\"", idev);
-				return -1;
-			}
+			idx = xll_name_to_index(idev);
 			addattr32(&req.n, sizeof(req), RTA_IIF, idx);
 		}
 		if (odev) {
-			if ((idx = ll_name_to_index(odev)) == 0) {
-				bb_error_msg("Cannot find device \"%s\"", odev);
-				return -1;
-			}
+			idx = xll_name_to_index(odev);
 			addattr32(&req.n, sizeof(req), RTA_OIF, idx);
 		}
 	}
@@ -758,7 +787,7 @@ static int iproute_get(int argc, char **argv)
 	}
 
 	if (rtnl_talk(&rth, &req.n, 0, 0, &req.n, NULL, NULL) < 0) {
-		exit(2);
+		return 2;
 	}
 
 	if (connected && !from_ok) {
@@ -766,18 +795,14 @@ static int iproute_get(int argc, char **argv)
 		int len = req.n.nlmsg_len;
 		struct rtattr * tb[RTA_MAX+1];
 
-		if (print_route(NULL, &req.n, (void*)stdout) < 0) {
-			bb_error_msg_and_die("An error :-)");
-		}
+		print_route(NULL, &req.n, (void*)stdout);
 
 		if (req.n.nlmsg_type != RTM_NEWROUTE) {
-			bb_error_msg("Not a route?");
-			return -1;
+			bb_error_msg_and_die("not a route?");
 		}
 		len -= NLMSG_LENGTH(sizeof(*r));
 		if (len < 0) {
-			bb_error_msg("Wrong len %d", len);
-			return -1;
+			bb_error_msg_and_die("wrong len %d", len);
 		}
 
 		memset(tb, 0, sizeof(tb));
@@ -787,8 +812,7 @@ static int iproute_get(int argc, char **argv)
 			tb[RTA_PREFSRC]->rta_type = RTA_SRC;
 			r->rtm_src_len = 8*RTA_PAYLOAD(tb[RTA_PREFSRC]);
 		} else if (!tb[RTA_SRC]) {
-			bb_error_msg("Failed to connect the route");
-			return -1;
+			bb_error_msg_and_die("failed to connect the route");
 		}
 		if (!odev && tb[RTA_OIF]) {
 			tb[RTA_OIF]->rta_type = 0;
@@ -803,31 +827,33 @@ static int iproute_get(int argc, char **argv)
 		req.n.nlmsg_type = RTM_GETROUTE;
 
 		if (rtnl_talk(&rth, &req.n, 0, 0, &req.n, NULL, NULL) < 0) {
-			exit(2);
+			return 2;
 		}
 	}
-
-	if (print_route(NULL, &req.n, (void*)stdout) < 0) {
-		bb_error_msg_and_die("An error :-)");
-	}
-
-	exit(0);
+	print_route(NULL, &req.n, (void*)stdout);
+	return 0;
 }
 
-int do_iproute(int argc, char **argv)
+/* Return value becomes exitcode. It's okay to not return at all */
+int do_iproute(char **argv)
 {
-	static const char * const ip_route_commands[] =
-		{ "add", "append", "change", "chg", "delete", "del", "get",
-		"list", "show", "prepend", "replace", "test", "flush", 0 };
-	int command_num = 7;
-	unsigned int flags = 0;
+	static const char ip_route_commands[] ALIGN1 =
+	/*0-3*/	"add\0""append\0""change\0""chg\0"
+	/*4-7*/	"delete\0""get\0""list\0""show\0"
+	/*8..*/	"prepend\0""replace\0""test\0""flush\0";
+	int command_num;
+	unsigned flags = 0;
 	int cmd = RTM_NEWROUTE;
 
-	if (*argv) {
-		command_num = compare_string_array(ip_route_commands, *argv);
-	}
-	switch(command_num) {
-		case 0: /* add*/
+	if (!*argv)
+		return iproute_list_or_flush(argv, 0);
+
+	/* "Standard" 'ip r a' treats 'a' as 'add', not 'append' */
+	/* It probably means that it is using "first match" rule */
+	command_num = index_in_substrings(ip_route_commands, *argv);
+
+	switch (command_num) {
+		case 0: /* add */
 			flags = NLM_F_CREATE|NLM_F_EXCL;
 			break;
 		case 1: /* append */
@@ -838,25 +864,27 @@ int do_iproute(int argc, char **argv)
 			flags = NLM_F_REPLACE;
 			break;
 		case 4: /* delete */
-		case 5: /* del */
 			cmd = RTM_DELROUTE;
 			break;
-		case 6: /* get */
-			return iproute_get(argc-1, argv+1);
-		case 7: /* list */
-		case 8: /* show */
-			return iproute_list_or_flush(argc-1, argv+1, 0);
-		case 9: /* prepend */
+		case 5: /* get */
+			return iproute_get(argv+1);
+		case 6: /* list */
+		case 7: /* show */
+			return iproute_list_or_flush(argv+1, 0);
+		case 8: /* prepend */
 			flags = NLM_F_CREATE;
-		case 10: /* replace */
+			break;
+		case 9: /* replace */
 			flags = NLM_F_CREATE|NLM_F_REPLACE;
-		case 11: /* test */
+			break;
+		case 10: /* test */
 			flags = NLM_F_EXCL;
-		case 12: /* flush */
-			return iproute_list_or_flush(argc-1, argv+1, 1);
+			break;
+		case 11: /* flush */
+			return iproute_list_or_flush(argv+1, 1);
 		default:
-			bb_error_msg_and_die("Unknown command %s", *argv);
+			bb_error_msg_and_die("unknown command %s", *argv);
 	}
 
-	return iproute_modify(cmd, flags, argc-1, argv+1);
+	return iproute_modify(cmd, flags, argv+1);
 }
