@@ -4,39 +4,34 @@
  *
  * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
  *
- * Licensed under the GPL v2, see the file LICENSE in this tarball.
+ * Licensed under the GPL version 2, see the file LICENSE in this tarball.
  */
 
-#include <stdio.h>
+#include "libbb.h"
 #include <mntent.h>
-#include <dirent.h>
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
-#include <sys/mount.h>
 #include <sys/swap.h>
 
-#include "busybox.h"
-
-static int swap_enable_disable(const char *device)
+static int swap_enable_disable(char *device)
 {
 	int status;
 	struct stat st;
 
 	xstat(device, &st);
 
+#if ENABLE_DESKTOP
 	/* test for holes */
 	if (S_ISREG(st.st_mode))
-		if (st.st_blocks * 512 < st.st_size)
-			bb_error_msg_and_die("swap file has holes");
+		if (st.st_blocks * (off_t)512 < st.st_size)
+			bb_error_msg("warning: swap file has holes");
+#endif
 
-	if (bb_applet_name[5] == 'n')
+	if (applet_name[5] == 'n')
 		status = swapon(device, 0);
 	else
 		status = swapoff(device);
 
 	if (status != 0) {
-		bb_perror_msg("%s", device);
+		bb_simple_perror_msg(device);
 		return 1;
 	}
 
@@ -63,8 +58,7 @@ static int do_em_all(void)
 	return err;
 }
 
-#define DO_ALL    0x01
-
+int swap_on_off_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int swap_on_off_main(int argc, char **argv)
 {
 	int ret;
@@ -72,11 +66,11 @@ int swap_on_off_main(int argc, char **argv)
 	if (argc == 1)
 		bb_show_usage();
 
-	ret = bb_getopt_ulflags(argc, argv, "a");
-	if (ret & DO_ALL)
+	ret = getopt32(argv, "a");
+	if (ret)
 		return do_em_all();
 
-	ret = 0;
+	/* ret = 0; redundant */
 	while (*++argv)
 		ret += swap_enable_disable(*argv);
 	return ret;
